# $Id: /mirror/coderepos/lang/perl/Data-Localize/trunk/t/lib/Test/Data/Localize/Namespace/en.pm 100685 2009-02-15T02:33:48.375991Z daisuke  $

package Test::Data::Localize::Namespace::en;

our %Lexicon = (
    'Hello, stranger!' => 'Hello, [_1]!'
);

1;